{*************
 *
 * Project....: CEO Library
 * File.......: SwExcept.pas
 * Author.....: Anton A. Mints
 * Date.......: 19 July 1997
 * Original...: STREAM LINE Office Application, 17 May 1996
 * Description: Form to display exception's information.
 *
 * $Archive: /LIB/FORMS/swexcept.pas $
 *
 * $History: swexcept.pas $
 * 
 * *****************  Version 3  *****************
 * User: Anton        Date: 7/21/97    Time: 12:41p
 * Updated in $/LIB/FORMS
 * Make comment's label empty by default.
 * 
 * *****************  Version 2  *****************
 * User: Anton        Date: 7/18/97    Time: 11:40p
 * Updated in $/LIB/FORMS
 * Remove form variable from interface part to avoid form autocreation by
 * mistake.
 * 
 * *****************  Version 1  *****************
 * User: Anton        Date: 7/18/97    Time: 11:36p
 * Created in $/lib/forms
 * Custom form to handle exception and show additional exception
 * information.
 *
 *************}

unit SwExcept;

interface

uses
  SysUtils, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls;

type
  TFrmException = class(TForm)
    pnlTop: TPanel;
    btnOk: TBitBtn;
    btnDetails: TButton;
    lblTechSupportMsg: TLabel;
    lblException: TLabel;
    Bevel3: TBevel;
    Label2: TLabel;
    pnlDetails: TPanel;
    Bevel2: TBevel;
    Bevel1: TBevel;
    lblEFuncName: TLabel;
    Label6: TLabel;
    Label4: TLabel;
    lblEClassName: TLabel;
    Label5: TLabel;
    lblEInModule: TLabel;
    Label1: TLabel;
    lblEComments: TLabel;
    procedure btnDetailsClick(Sender: TObject);

  private
    EMessage: String;
    EFuncName: String;

    function GetTechSupportMsg: String;
    procedure SetTechSupportMsg (Value: String);

    procedure ShowDetailsInfo;

    procedure DetailsShow;
    procedure DetailsHide;

  public
    constructor Create(AOwner: TComponent); override;
    procedure CustomizedHandler (Sender: TObject; E: Exception);

    property TechSupportMsg: String read GetTechSupportMsg
      write SetTechSupportMsg;
  end;


implementation

uses
  Mask, CEOExcpt;

{$R *.DFM}

const
  TAG_SHOW_DETAILS = 1;
  TAG_HIDE_DETAILS = 2;

var
  FrmException: TFrmException;

{*******************************************************************}
{*                      Form TFrmException                         *}
{*******************************************************************}

constructor TFrmException.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  // Put Details panel on the proper place
  pnlDetails.Top := pnlTop.Height;
end;


{*************
* Name.......: CustomizedHandler
*}
procedure TFrmException.CustomizedHandler (Sender: TObject; E: Exception);
var
  EPos, ELen: Integer;

begin
  // If the data is not compatible with the mask specified for
  //the field display default form because it's actualy
  //just an error message.
  if ( E is EDBEditError ) then
  begin
    // Show default exception form
    Application.ShowException (E);
    // ...and exit
    Exit;
  end;

  // If it's our special exception, easly extract the comment string
  if ( ExceptObject is ECEOError ) then
    begin
      EMessage := E.Message;
      EFuncName := ECEOError(ExceptObject).CallPath;
    end

  // else try to process the line according with lod agreements
  // [Procedure Name: Text of comments...]
  else
    begin
      EFuncName := '';
      EMessage := E.Message;
      EPos := Pos(': ', EMessage);
      ELen := Length (EMessage);

      if ( (EPos > 0) and (EPos < ELen - 2) ) then
      begin
        EFuncName := Copy(EMessage, 1, EPos + 1);
        EMessage := Copy(EMessage, EPos + 2, ELen - EPos - 1);
      end;
    end;

  // Display exception message on the form
  lblException.Caption := EMessage;

  // Don't show detail part by default
  DetailsHide;

  // Show the form
  ShowModal;
end;


{*************
* Name.......: ShowDetailsInfo
*}
procedure TFrmException.ShowDetailsInfo;

  {*************
  * Name.......: GetModuleName
  *}
  function GetModuleName: String;
  var
    Buffer: array[0..255] of Char;
  begin
    GetModuleFileName (HInstance, Buffer, SizeOf(Buffer));
    Result := ExtractFileName (String(Buffer));
  end;

  {*************
  * Name.......: ConvertAddr
  * Description: Convert physical address to logical address.
  *}
  function ConvertAddr (Address: Pointer): Pointer; assembler;
  asm
    TEST    EAX, EAX         { Always convert nil to nil }
    JE      @@1
    SUB     EAX, OFFSET TextStart
  @@1:
  end;

begin
  lblEFuncName.Caption := EFuncName;
  lblEClassName.Caption := ExceptObject.ClassName;
  lblEInModule.Caption := Format ('%s at %p', [GetModuleName, ConvertAddr (ExceptAddr)]);

  // Our special exseption class can keep comments additionaly
  if ( ExceptObject is ECEOError ) then
    lblEComments.Caption := ECEOError(ExceptObject).Comment
  else
    lblEComments.Caption := '';
end;


{*************
* Name.......: GetTechSupportMsg
*}
function TFrmException.GetTechSupportMsg: String;
begin
  Result := lblTechSupportMsg.Caption;
end;


{*************
* Name.......: SetTechSupportMsg
*}
procedure TFrmException.SetTechSupportMsg (Value: String);
begin
  lblTechSupportMsg.Caption := Value;
end;


{*************
* Name.......: DetailsShow
*}
procedure TFrmException.DetailsShow;
begin
  ClientHeight := pnlTop.Height + pnlDetails.Height;

  btnDetails.Caption := '&Hide Details <<<';
  btnDetails.Tag := TAG_HIDE_DETAILS;

  ShowDetailsInfo;
end;


{*************
* Name.......: DetailsHide
*}
procedure TFrmException.DetailsHide;
begin
  ClientHeight := pnlTop.Height;

  btnDetails.Caption := '&Show Details >>>';
  btnDetails.Tag := TAG_SHOW_DETAILS;
end;


{*************
* Name.......: btnDetails (OnClick EVENT)
*}
procedure TFrmException.btnDetailsClick(Sender: TObject);
begin
  case btnDetails.Tag of
    TAG_SHOW_DETAILS: DetailsShow;
    TAG_HIDE_DETAILS: DetailsHide;
  else
    RaiseCEOError (nil,
      ['TFrmException.btnDetailsClick', 'Tag is not defined.']);
  end;
end;


initialization
  // Create exception form
  frmException := TFrmException.Create (Application);
  // Change default exception form handler
  Application.OnException := frmException.CustomizedHandler;
end.


